@@ -6,7 +6,7 @@
 import numpy as np
 
 # Project imports
-from weac.tools import gerling, calc_center_of_gravity, load_dummy_profile
+from weac.tools import bergfeld, calc_center_of_gravity, load_dummy_profile
 
 
 class Eigensystem:
@@ -90,17 +90,18 @@ class Eigensystem:
         Describes the stiffnesses of weak-layer and slab.
     """
 
-    def __init__(self, system='pst-'):
+    def __init__(self, system='pst-', touchdown=False):
         """
         Initialize eigensystem with user input.
 
         Arguments
         ---------
-        system : {'pst-', '-pst', 'skier', 'skiers'}, optional
+        system : {'pst-', '-pst', 'vpst-', '-vpst', 'skier', 'skiers'}, optional
             Type of system to analyse: PST cut from the right (pst-),
-            PST cut form the left (-pst), one skier on infinite
-            slab (skier) or multiple skiers on infinite slab (skeirs).
-            Default is 'pst-'.
+            PST cut form the left (-pst), PST with vertical faces cut
+            from the right (vpst-), PST with vertical faces cut from the
+            left (-vpst), one skier on infinite slab (skier) or multiple
+            skiers on infinite slab (skiers). Default is 'pst-'.
         layers : list, optional
             2D list of layer densities and thicknesses. Columns are
             density (kg/m^3) and thickness (mm). One row corresponds
@@ -110,44 +111,51 @@ class Eigensystem:
         self.g = 9810           # Gravitaiton (mm/s^2)
         self.lski = 1000        # Effective out-of-plane length of skis (mm)
         self.tol = 1e-3         # Relative Romberg integration tolerance
-        self.system = system    # 'pst-', '-pst', 'skier', 'skiers'
+        self.system = system    # 'pst-', '-pst', 'vpst-', '-vpst', 'skier', 'skiers'
 
         # Initialize weak-layer attributes that will be filled later
-        self.weak = False       # Weak-layer properties dictionary
-        self.t = False          # Weak-layer thickness (mm)
-        self.kn = False         # Weak-layer compressive stiffness
-        self.kt = False         # Weak-layer shear stiffness
-        self.tc = False         # Weak-layer collapse height (mm)
+        self.weak = False           # Weak-layer properties dictionary
+        self.t = False              # Weak-layer thickness (mm)
+        self.kn = False             # Weak-layer compressive stiffness
+        self.kt = False             # Weak-layer shear stiffness
 
         # Initialize slab attributes
-        self.p = 0              # Surface line load (N/mm)
-        self.slab = False       # Slab properties dictionary
-        self.k = False          # Slab shear correction factor
-        self.h = False          # Total slab height (mm)
-        self.zs = False         # Z-coordinate of slab center of gravity (mm)
-        self.phi = False        # Slab inclination (°)
-        self.A11 = False        # Slab extensional stiffness
-        self.B11 = False        # Slab bending-extension coupling stiffness
-        self.D11 = False        # Slab bending stiffness
-        self.kA55 = False       # Slab shear stiffness
-        self.K0 = False         # Stiffness determinant
+        self.p = 0                  # Surface line load (N/mm)
+        self.slab = False           # Slab properties dictionary
+        self.k = False              # Slab shear correction factor
+        self.h = False              # Total slab height (mm)
+        self.zs = False             # Z-coordinate of slab center of gravity (mm)
+        self.phi = False            # Slab inclination (°)
+        self.A11 = False            # Slab extensional stiffness
+        self.B11 = False            # Slab bending-extension coupling stiffness
+        self.D11 = False            # Slab bending stiffness
+        self.kA55 = False           # Slab shear stiffness
+        self.K0 = False             # Stiffness determinant
 
         # Inizialize eigensystem attributes
-        self.ewC = False        # Complex eigenvalues
-        self.ewR = False        # Real eigenvalues
-        self.evC = False        # Complex eigenvectors
-        self.evR = False        # Real eigenvectors
-        self.sC = False         # Stability shift of complex eigenvalues
-        self.sR = False         # Stability shift of real eigenvalues
+        self.ewC = False            # Complex eigenvalues
+        self.ewR = False            # Real eigenvalues
+        self.evC = False            # Complex eigenvectors
+        self.evR = False            # Real eigenvectors
+        self.sC = False             # Stability shift of complex eigenvalues
+        self.sR = False             # Stability shift of real eigenvalues
 
         # Initialize touchdown attributes
-        self.lC = False         # Minimum length of substratum contact (mm)
-        self.lS = False         # Maximum length of span between
-                                # between bedded and touchdowned boundary (mm)
-        self.ratio = False      # Stiffness ratio of collalpsed to uncollapsed weak-layer
-        self.beta = False       # Ratio of slab to bedding stiffness
+        self.touchdown = touchdown  # Flag whether touchdown is possible
+        self.a = False              # Cracklength
+        self.tc = False             # Weak-layer collapse height (mm)
+        self.ratio = False          # Stiffness ratio of collapsed to uncollapsed weak-layer
+        self.betaU = False          # Ratio of slab to bedding stiffness (uncollapsed)
+        self.betaC = False          # Ratio of slab to bedding stiffness (collapsed)
+        self.mode = False           # Touchdown-mode can be either A, B, C or D
+        self.td = False             # Touchdown length
 
-    def set_foundation_properties(self, t=30, cf=0.5, E=0.25, nu=0.25, update=False):
+    def set_foundation_properties(
+            self,
+            t: float = 10.0,
+            E: float = 0.25,
+            nu: float = 0.25,
+            update: bool = False):
         """
         Set material properties and geometry of foundation (weak layer).
 
@@ -168,7 +176,6 @@ class Eigensystem:
         """
         # Geometry
         self.t = t              # Weak-layer thickness (mm)
-        self.tc = cf*self.t     # Weak-layer collapse height (mm)
 
         # Material properties
         self.weak = {
@@ -180,7 +187,7 @@ class Eigensystem:
         if update:
             self.calc_fundamental_system()
 
-    def set_beam_properties(self, layers, phi=0, C0=6.0, C1=4.60,
+    def set_beam_properties(self, layers, phi=0, C0=6.5, C1=4.4,
                             nu=0.25, update=False):
         """
         Set material and properties geometry of beam (slab).
@@ -192,14 +199,12 @@ class Eigensystem:
             Columns are density (kg/m^3) and thickness (mm). One row
             corresponds to one layer. If entered as str, last split
             must be available in database.
-        phi : float
-            Inclination of the slab (degrees).
         C0 : float, optional
             Multiplicative constant of Young modulus parametrization
-            according to Gerling et al. (2017). Default is 6.0.
+            according to Bergfeld et al. (2023). Default is 6.0.
         C1 : float, optional
             Exponent of Young modulus parameterization according to
-            Gerling et al. (2017). Default is 4.6.
+            Bergfeld et al. (2023). Default is 4.6.
         nu : float, optional
             Possion's ratio. Default is 0.25
         update : bool, optional
@@ -212,7 +217,7 @@ class Eigensystem:
         else:
             # Compute Young's modulus from density parametrization
             layers = np.array(layers)
-            E = gerling(layers[:, 0], C0=C0, C1=C1)  # Young's modulus
+            E = bergfeld(layers[:, 0], C0=C0, C1=C1)  # Young's modulus
 
         # Derive other elastic properties
         nu = nu*np.ones(layers.shape[0])         # Global poisson's ratio
@@ -221,16 +226,13 @@ class Eigensystem:
 
         # Compute total slab thickness and center of gravity
         self.h, self.zs = calc_center_of_gravity(layers)
-
+        
         # Assemble layering into matrix (top to bottom)
         # Columns are density (kg/m^3), layer thickness (mm)
         # Young's modulus (MPa), shear modulus (MPa), and
         # Poisson's ratio
         self.slab = np.vstack([layers.T, E, G, nu]).T
 
-        # Set beam inclination
-        self.phi = phi
-
         # Recalculate the fundamental system after properties have changed
         if update:
             self.calc_fundamental_system()
@@ -252,7 +254,7 @@ class Eigensystem:
         """
         self.p = p
 
-    def calc_foundation_stiffness(self, ratio=16):
+    def calc_foundation_stiffness(self):
         """Compute foundation normal and shear stiffness."""
         # Elastic moduli (MPa) under plane-strain conditions
         G = self.weak['E']/(2*(1 + self.weak['nu']))    # Shear modulus
@@ -262,9 +264,6 @@ class Eigensystem:
         self.kn = E/self.t                              # Normal stiffness
         self.kt = G/self.t                              # Shear stiffness
 
-        # Weak-layer stiffness increment factor for collapse
-        self.ratio = ratio
-
     def get_ply_coordinates(self):
         """
         Calculate ply (layer) z-coordinates.
@@ -478,142 +477,6 @@ class Eigensystem:
 
         return Fn, Ft
 
-    def calc_beta(self):
-        """
-        Calculate beta.
-
-        Returns
-        -------
-        beta : float
-            Weak-layer to slab stiffness relation factor.
-        """
-        # (Intact) weak-layer to slab stiffness relation factor
-        self.beta = (self.kn/(4*self.D11))**(1/4)
-
-    def calc_span_length(self):
-        """
-        Calculate span from layer and weak layer properties and load situation.
-
-        Returns
-        -------
-        lS : float
-            Span of the element between bedded element and touchdown for full touchdown.
-        """
-        def polynomial():
-            """
-            Calculate the coefficients of a sixth order polynomial equation.
-
-            Returns
-            -------
-            list
-                First coefficient for sixth order term,
-                second coefficient for fith order term and so on.
-            """
-            a1 = self.kA55**2*kR1*kN1*q0
-            a2 = 6*self.kA55*(self.D11*self.kA55 + kR1*kR2)*kN1*q0
-            a3 = 30*self.D11*self.kA55*(kR1 + kR2)*kN1*q0
-            a4 = 24*self.D11*(2*self.kA55**2*kR1 + 3*self.D11*self.kA55*kN1 + 3*kR1*kR2*kN1)*q0
-            a5 = 72*self.D11*(self.D11*(self.kA55**2 + (kR1 + kR2)*kN1)*q0 \
-                + self.kA55*kR1*(2*kR2*q0 - self.kA55*kN1*self.tc))
-            a6 = 144*self.D11*self.kA55*(self.D11*(kR1 + kR2)*q0 \
-                - (self.D11*self.kA55 + kR1*kR2)*kN1*self.tc)
-            a7 = - 144*self.D11**2*self.kA55*(kR1 + kR2)*kN1*self.tc
-            return [a1,a2,a3,a4,a5,a6,a7]
-
-        # Get spring stiffnesses for adjacent segment with intact weak-layer
-        kR1 = self.calc_rot_spring(collapse=False)
-        kN1 = self.calc_trans_spring()
-        # Get spring stiffnesses for adjacent segment with collapsed weak-layer
-        kR2 = self.calc_rot_spring(collapse=True)
-        # Get surface normal load components
-        qn = self.get_weight_load(self.phi)[0]
-        pn = self.get_surface_load(self.phi)[0]
-        q0 = qn + pn
-        # Calculate positive real roots
-        pos = (np.roots(polynomial()).imag == 0) & (np.roots(polynomial()).real > 0)
-        self.lS = np.roots(polynomial())[pos].real[0]
-
-    def calc_contact_length(self):
-        """
-        Calculate segment length where max slab deflection equals tc.
-
-        Returns
-        -------
-        lC : float
-            Maximum length without substratum contact.
-        """
-        def polynomial():
-            """
-            Calculate the coefficients of a fourth order polynomial equation.
-
-            Returns
-            -------
-            list
-                First coefficient for fourth order term,
-                second coefficient for third order term and so on.
-            """
-            a1 = 1/(8*self.D11)*q0
-            a2 = 1/(2*kR1)*q0
-            a3 = 1/(2*self.kA55)*q0
-            a4 = 1/kN1*q0
-            a5 = -self.tc
-            return [a1,a2,a3,a4,a5]
-
-        # Get spring stiffnesses for adjacent segment intact intact weak-layer
-        kR1 = self.calc_rot_spring(collapse=False)
-        kN1 = self.calc_trans_spring()
-        # Get surface normal load components
-        qn = self.get_weight_load(self.phi)[0]
-        pn = self.get_surface_load(self.phi)[0]
-        q0 = qn + pn
-        # Calculate positive real roots
-        pos = (np.roots(polynomial()).imag == 0) & (np.roots(polynomial()).real > 0)
-        self.lC = np.roots(polynomial())[pos].real[0]
-
-    def calc_rot_spring(self, collapse=True):
-        """
-        Calculate rotational spring stiffness from layer properties.
-
-        Arguments
-        ---------
-        collapse : boolean
-            Indicates whether weak-layer is collapsed.
-
-        Returns
-        -------
-        kR : float
-            Rotational spring stiffness (Nmm/mm/rad).
-        """
-        # get ratio for foundation stiffness after collapse
-        if collapse:
-            ratio = self.ratio
-        else:
-            ratio = 1
-        # calc spring stiffness
-        kR = self.D11*self.beta*ratio**(1/4)
-
-        return kR
-
-    def calc_trans_spring(self):
-        """
-        Calculate translational spring stiffness from layer properties.
-
-        Returns
-        -------
-        kN : float
-            Translational spring stiffness (N/mm^2).
-        """
-        # calc translational spring stiffness for bedded euler-bernoulli-beam
-        kN = 2*self.D11*self.beta**3
-
-        return kN
-
-    def calc_touchdown_system(self):
-        """Calculate the lenghts for touchdown evaluation"""
-        self.calc_beta()
-        self.calc_span_length()
-        self.calc_contact_length()
-
     def zh(self, x, l=0, bed=True):
         """
         Compute bedded or free complementary solution at position x.
@@ -692,7 +555,7 @@ class Eigensystem:
         A11 = self.A11
         B11 = self.B11
         kA55 = self.kA55
-        E0 = self.K0
+        K0 = self.K0
 
         # Unpack geometric properties
         h = self.h
@@ -711,13 +574,13 @@ class Eigensystem:
                 [0]])
         else:
             zp = np.array([
-                [(-3*(qt + pt)/A11 - B11*(qn + pn)*x/E0)/6*x**2],
-                [(-2*(qt + pt)/A11 - B11*(qn + pn)*x/E0)/2*x],
-                [-A11*(qn + pn)*x**4/(24*E0)],
-                [-A11*(qn + pn)*x**3/(6*E0)],
-                [A11*(qn + pn)*x**3/(6*E0)
+                [(-3*(qt + pt)/A11 - B11*(qn + pn)*x/K0)/6*x**2],
+                [(-2*(qt + pt)/A11 - B11*(qn + pn)*x/K0)/2*x],
+                [-A11*(qn + pn)*x**4/(24*K0)],
+                [-A11*(qn + pn)*x**3/(6*K0)],
+                [A11*(qn + pn)*x**3/(6*K0)
                  + ((zs - B11/A11)*qt - h*pt/2 - (qn + pn)*x)/kA55],
-                [(qn + pn)*(A11*x**2/(2*E0) - 1/kA55)]])
+                [(qn + pn)*(A11*x**2/(2*K0) - 1/kA55)]])
 
         return zp
 
